package consumer

import (
	"context"
	"encoding/json"
	"errors"
	"fmt"
	"fusenapi/constants"
	"fusenapi/initalize"
	"fusenapi/model/gmodel"
	"fusenapi/utils/websocket_data"
	"github.com/zeromicro/go-zero/core/logx"
	"gorm.io/gorm"
	"strconv"
)

// 这里请求的py接口返回数据
type pythonApiRsp struct {
	Code int    `json:"code"`
	Msg  string `json:"msg"`
	Data []struct {
		Tid      int64  `json:"tid"`
		Imgurl   string `json:"imgurl"`
		Costtime int64  `json:"costtime"`
	} `json:"data"`
}

// 消费渲染需要组装的数据
type MqConsumerRenderAssemble struct {
}

func (m *MqConsumerRenderAssemble) Run(ctx context.Context, data []byte) error {
	logx.Info("收到需要组装的消息:", string(data))
	var parseInfo websocket_data.AssembleRenderData
	if err := json.Unmarshal(data, &parseInfo); err != nil {
		logx.Error("MqConsumerRenderAssemble数据格式错误:", err)
		return nil //不返回错误就删除消息
	}
	val := ctx.Value("allmodels")
	if val == nil {
		return errors.New("allmodels is nil")
	}
	allmodels, ok := val.(*gmodel.AllModelsGen)
	if !ok {
		return errors.New("allmodels is nil!!")
	}
	rabbitmq := initalize.RabbitMqHandle{}
	//获取模板
	templateInfo, err := allmodels.FsProductTemplateV2.FindOneByProductIdTagIdWithSizeTable(ctx, parseInfo.RenderData.ProductId, fmt.Sprintf("%d", parseInfo.RenderData.TemplateTagId))
	if err != nil {
		if errors.Is(err, gorm.ErrRecordNotFound) {
			logx.Error("template info is not found")
			return nil
		}
		logx.Error("failed to get template info:", err)
		return err
	}
	// todo curl请求python获取刀版图 baseImage######
	baseImage := ""
	//获取渲染设置信息
	element, err := allmodels.FsProductTemplateElement.FindOneByModelId(ctx, *templateInfo.ModelId)
	if err != nil {
		if errors.Is(err, gorm.ErrRecordNotFound) {
			logx.Error("element info is not found,model_id = ?", *templateInfo.ModelId)
			return nil
		}
		logx.Error("failed to get element list,", err)
		return err
	}
	//组装数据
	refletion := -1
	if element.Refletion != nil && *element.Refletion != "" {
		refletion, err = strconv.Atoi(*element.Refletion)
	}
	//组装data数据
	var mode map[string]interface{}
	if element.Mode != nil && *element.Mode != "" {
		if err = json.Unmarshal([]byte(*element.Mode), &mode); err != nil {
			logx.Error("faile to parse element mode json:", err)
			return err
		}
	}
	tempData := make([]map[string]interface{}, 0, 3)
	if element.Base != nil && *element.Base != "" {
		tempData = append(tempData, map[string]interface{}{
			"name":      "model",
			"data":      "0," + baseImage + "," + *element.Base,
			"type":      "other",
			"layer":     "0",
			"is_update": 1,
			"mode":      mode["model"],
		})
	}
	if element.Shadow != nil && *element.Shadow != "" {
		tempData = append(tempData, map[string]interface{}{
			"name":      "shadow",
			"data":      *element.Shadow,
			"type":      "other",
			"layer":     "0",
			"is_update": 0,
			"mode":      mode["shadow"],
		})
	}
	if element.ModelP != nil && *element.ModelP != "" {
		tempData = append(tempData, map[string]interface{}{
			"name":      "model_P",
			"data":      "0," + *element.ModelP,
			"type":      "other",
			"layer":     "0",
			"is_update": 0,
			"mode":      mode["model_P"],
		})
	}
	result := []interface{}{
		map[string]interface{}{
			"light":     *element.Light,
			"refletion": refletion,
			"scale":     *element.Scale,
			"sku_id":    *templateInfo.ProductId,
			"tid":       *element.Title,
			"rotation":  *element.Rotation,
			"filePath":  "", //todo 文件路径,针对千人千面
			"data":      tempData,
		},
	}
	sendData := map[string]interface{}{
		"id":               parseInfo.TaskId,
		"order_id":         0,
		"user_id":          parseInfo.RenderData.UserId,
		"guest_id":         parseInfo.RenderData.GuestId,
		"sku_ids":          []int64{parseInfo.RenderData.ProductId},
		"tids":             []string{*element.Title},
		"data":             result,
		"is_thousand_face": 0,
		"folder":           "", //todo 千人千面需要使用
	}
	b, _ := json.Marshal(sendData)
	if err = rabbitmq.SendMsg(constants.RABBIT_MQ_TO_UNITY, b); err != nil {
		logx.Error("发送渲染组装数据到rabbitmq失败：", err)
		return err
	}
	logx.Info("发送渲染组装数据到unity成功")
	return nil
}
